//===================================================== file = generate.c =====
//=  Traffic generator for output buffered switch simulation model            =
//=============================================================================
//=  Notes:  1)                                                               =
//=---------------------------------------------------------------------------=
//=  Contains: gen_poisson() - Poisson with IEEE workgroup mix                =
//=            gen_ipp() - IPP w/ fixed packet length                         =
//=            gen_cbr() - CBR w/ fixed packet length and rate                =
//=            gen_square() - Square wave w/ fixed packet length and period   =
//=---------------------------------------------------------------------------=
//=  History:  KJC (05/07/01) - Genesis                                       =
//=============================================================================
#define EXTERNAL        // This is an external file (used in common.h)

//----- Include files ---------------------------------------------------------
#include "csim.h"       // Needed for CSIM
#include "common.h"     // Needed for common stuff

//============================================================================
//==  Function to generate IEEE workstation mix packets (Poisson arrivals)  ==
//==    - Based on "WG AVG" in Frazier document                             ==
//==    - Assume that "< 1600" is same as "< 1500" in Frazier document      ==
//==    - Mean packet length is 590.1 bytes                                 ==
//============================================================================
void gen_poisson(int in_id)
{
  int    out_id;        // To (destination) outport number
  int    pkt_len;       // Packet length pulled from IEEE dist
  double z;             // Random number between 0.0 and 1.0

  create("gen_poisson");

  // Packet generation loop
  while(TRUE)
  {
    // Hold for desired interarrival time
    hold(expntl(1.0 / Lambda1));

    // Choose packet size in bytes from empirical IEEE distribution
    z = uniform(0.0, 1.0);
    if (z < 0.197) pkt_len = 100;
    else if (z < 0.406) pkt_len = 200;
    else if (z < 0.464) pkt_len = 300;
    else if (z < 0.498) pkt_len = 400;
    else if (z < 0.519) pkt_len = 500;
    else if (z < 0.628) pkt_len = 600;
    else if (z < 0.647) pkt_len = 700;
    else if (z < 0.667) pkt_len = 800;
    else if (z < 0.687) pkt_len = 900;
    else if (z < 0.719) pkt_len = 1000;
    else if (z < 0.915) pkt_len = 1100;
    else if (z < 0.916) pkt_len = 1200;
    else if (z < 0.917) pkt_len = 1300;
    else if (z < 0.919) pkt_len = 1400;
    else if (z <= 1.000) pkt_len = 1500;

    // Choose the output port
    out_id = random_int(0, (N - 1));

    // Send the packet
    in[in_id](in_id, out_id, pkt_len, clock);
  }
}

//============================================================================
//==  Function to generate IPP packets of constant length                   ==
//==    - All packets in an "on" period are destined for the same output    ==
//==    - Packet length parameter is "wired" into the function              ==
//============================================================================
void gen_ipp(int in_id)
{
  int    out_id;        // To (destination) outport number
  int    pkt_len;       // Packet length
  double z;             // Random number between 0.0 and 1.0

  create("gen_ipp");

  // Set the packet length and type
  pkt_len = 1500;

  // Choose the initial output port
  out_id = random_int(0, (N - 1));

  // Packet generation loop
  while(TRUE)
  {
    // Hold for a packet interarrival time
    hold(expntl(1.0 / (Lambda2 + Alpha)));

    // Send the packet or else do an off period and choose a new output for
    // the next on period
    z = uniform(0.0, 1.0);
    if (z < (Lambda2 / (Lambda2 + Alpha)))
    {
      in[in_id](in_id, out_id, pkt_len, clock);
    }
    else
    {
      hold(expntl(1.0 / Beta));
      out_id = random_int(0, (N - 1));
    }
  }
}

//============================================================================
//==  Function to generate CBR packets of constant length                   ==
//==    - Rate and packet length parameters are "wired" into the function   ==
//============================================================================
void gen_cbr(int in_id)
{
  int    out_id;        // To (destination) outport number
  int    pkt_len;       // Packet length
  double pkt_rate;      // Packet rate

  create("gen_cbr");

  // Set the packet rate, length, and type
  pkt_rate = 833.33333333;
  pkt_len = 1500;

  // Packet generation loop
  while(TRUE)
  {
    // Hold for desired interarrival time
    hold(1.0 / pkt_rate);

    // Choose the output port
    out_id = random_int(0, (N - 1));

    // Send the packet
    in[in_id](in_id, out_id, pkt_len, clock);
  }
}

//============================================================================
//==  Function to generate "square wave" of packets of constant length      ==
//==    - Packet length, on count, and off count parameters are "wired"     ==
//==      into the function                                                 ==
//============================================================================
void gen_square(int in_id)
{
  int    out_id;        // To (destination) outport number
  int    pkt_len;       // Packet length
  int    on_period;     // Number of packets to send in an on period
  int    off_period;    // Off period count
  int    i;             // Loop counter

  create("gen_square");

  // Set the packet length, type, on_period, and off_count
  pkt_len = 1500;
  on_period = 10;
  off_period = 10;

  // Packet generation loop
  while(TRUE)
  {
    // Choose the output port and do the on period
    out_id = random_int(0, (N - 1));
    for (i=0; i<on_period; i++)
    {
      // Hold for one full packet time
      hold ((8.0 * pkt_len) / MEDIA_RATE);

      // Send the packet
      in[in_id](in_id, out_id, pkt_len, clock);
    }

    // Do the off period
    for (i=0; i<off_period; i++)
    {
      // Hold for one full packet time
      hold ((8.0 * pkt_len) / MEDIA_RATE);
    }
  }
}
