//========================================================= file = main.c =====
//=  Main program for output buffered switch simulation model                 =
//=============================================================================
//=  Notes:  1) Simulation output is written to created file OUTFILE defined  =
//=             in common.h                                                   =
//=---------------------------------------------------------------------------=
//=  Build:  Build as a CSIM application linking to appropriate CSIM libs.    =
//=          Component files are:                                             =
//=            - common.h, main.c, generate.c, inport.c, outport.c            =
//=---------------------------------------------------------------------------=
//=  History:  KJC (06/12/00) - Genesis                                       =
//=            KJC (06/30/00) - Added IBP generator                           =
//=            KJC (05/07/01) - Cleaned-up to create 1.00                     =
//=============================================================================
#define MAIN                    // This is main (used in common.h)

//----- Include files ---------------------------------------------------------
#include <stdio.h>              // Needed for printf() and fprintf()
#include <stdlib.h>             // Needed for exit()
#include <math.h>               // Needed for sqrt()
#include "csim.h"               // Needed for CSIM
#include "common.h"             // Needed for common stuff

//=============================================================================
//==  Main program where sim() is the first process for a CSIM simulation    ==
//=============================================================================
void sim(void)
{
  FILE     *fp;                 // File pointer for output file
  int      run_num;             // Simulation run number
  double   global_mean;         // Overall (global) mean delay
  double   global_std_dev;      // Overall (global) std dev delay
  int      i;                   // Generic loop counter

  // Set model name for CSIM
  set_model_name("Output buffered switch simulation model");

  // Increase maximum number of allowed processes, facilities, and servers
  max_facilities(1000L);
  max_servers(10000L);
  max_processes(1000000L);
  max_events(1000000L);

  // Open output file for writing (delete and re-write if file already exists)
  fp = fopen(OUT_FILE, "w");
  if (fp == NULL)
  {
    printf("  >>> ERROR in creating output file '%s' \n", OUT_FILE);
    exit(1);
  }

  // Output banner to stdout and output file
  printf("*** BEGIN SIMULATION -- Version %s *** \n", VERSION);
  fprintf(fp,"*** BEGIN SIMULATION -- Version %s *** \n", VERSION);

  // *** Main simulation loop ***
  for (run_num=9; run_num<10; run_num++)
  {
    create("sim");

    // Initialize run parameters (as appropriate for generate1 or generate2)
    Sim_time = 1000.0;
    Lambda1 = (run_num / 10.0) * 2118.2850;

    // Assign pointers to generators, inports, and outports
    for (i=0; i<N; i++)
    {
      if (GEN_FLAG == 1) gen[i] = gen_poisson;
      if (GEN_FLAG == 2) gen[i] = gen_ipp;
      if (GEN_FLAG == 3) gen[i] = gen_cbr;
      if (GEN_FLAG == 4) gen[i] = gen_square;
      in[i]  = inport;
      out[i] = outport;
    }

    // CSIM initializations
    for (i=0; i<N; i++)
    {
      Q_server[i] = facility("Server for an output queue");
      Bytes_table[i] = table("Bytes transfered table");
      Delay_table[i] = table("Packet delay table");
    }

    // Kick-off the generators
    for (i=0; i<N; i++)
      gen[i](i);

    // Run until Sim_time expires while outputting status messages to stdout
    run_sim(Sim_time, run_num);

    // Compute global statistics
    global_mean = global_std_dev = 0.0;
    for (i=0; i<N; i++)
    {
      global_mean = global_mean + table_mean(Delay_table[i]) / N;
      global_std_dev = global_std_dev + sqrt(table_var(Delay_table[i])) / N;
    }

    // Output simulation configuration parameters
    fprintf(fp,"&*************************************************\n");
    fprintf(fp,"&** Results from output buffered packet switch  **\n");
    fprintf(fp,"&*************************************************\n");
    if (GEN_FLAG == 1)
    {
      fprintf(fp,"&**   >>> Poisson with IEEE workgroup mix       \n");
      fprintf(fp,"&** Arrival rate        = %5.2f pkts/ms         \n",
        Lambda1);
    }
    if (GEN_FLAG == 2)
      fprintf(fp,"&**   >>> IPP with fixed packet length          \n");
    if (GEN_FLAG == 3)
      fprintf(fp,"&**   >>> CBR with fixed packet length          \n");
    if (GEN_FLAG == 4)
      fprintf(fp,"&**   >>> Square wave with fixed packet length  \n");
    fprintf(fp,"&** Simulation time      = %7.0f ms               \n",
      Sim_time);
    fprintf(fp,"&** Media rate           = %9.0f Mbps             \n",
      0.001 * MEDIA_RATE);
    fprintf(fp,"&** Switch size          = %d ports               \n",
      N);
    fprintf(fp,"&************************************************\n");

    // Output per port statistics
    for (i=0; i<N; i++)
    {
      fprintf(fp,"%2d====================================================\n",
        i);
      fprintf(fp,"%2d= Outport utilization  = %5.2f %%       \n", i,
        100.0 * (8.0 * table_sum(Bytes_table[i])) / (MEDIA_RATE * Sim_time));
      fprintf(fp,"%2d= Mean delay           = %f ms          \n", i,
        table_mean(Delay_table[i]));
      fprintf(fp,"%2d= Std Dev delay        = %f ms          \n", i,
        sqrt(table_var(Delay_table[i])));
    }

    // Output global statistics
    fprintf(fp,"&&====================================================\n");
    fprintf(fp,"&&  global mean delay    = %f ms \n", global_mean);
    fprintf(fp,"&&  global std dev delay = %f ms \n", global_std_dev);
    fprintf(fp,"&&====================================================\n");

    // Reset everything for a possible rerun
    rerun();
    reset_prob(1L);
  }

  // Close and end
  printf("*** END OF SIMULATION *** \n");
  fprintf(fp,"*** END OF SIMULATION *** \n");
  fclose(fp);
}
